/* Name: main.c
 * Project: PowerSwitch based on AVR USB driver
 * Author: Christian Starkjohann
 * Creation Date: 2005-01-16
 * Tabsize: 4
 * Copyright: (c) 2005 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt) or proprietary (CommercialLicense.txt)
 * This Revision: $Id: main.c 523 2008-02-15 09:46:40Z cs $
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>

#include "usbdrv.h"
#include "oddebug.h"
#include <util/delay.h>

#include <stdint.h>

/* USB Request-Commands */
#define PSCMD_CALL_ECHO		0
#define PSCMD_CALL_GETSTATUS	1

/* USB Response-Codes */
#define PSCMD_RETN_OKAY		(0x0)
#define PSCMD_RETN_BUSY		(0x1)
#define PSCMD_RETN_UNKNOW	(0xff)


/* Mainprogramm states (mainstate) */
#define STATE_AFTERRESET	0
#define STATE_BEGINNING		1
#define STATE_INITIATED		2

static volatile uint32_t	jiffies;	     /* more or less 1ms per passed jiffy */
static volatile uint8_t		inintterupt;	     /* signal usbPoll to be called from interrupt */

static volatile uint8_t		mainstate;





/*   **TIMER FUNCTIONS**    */

#define timerstop() (TCCR0 &= ~(0x7)) /* timer0 aus */
#define timerstop_safe(bakup) ({ bakup=(TCCR0 & 0x7); timerstop(); }) /* timer0 aus */

#define timercont(bakup) (TCCR0 |= bakup) /* timer0 aus */
#define timerrestart() (TCNT0 = 0) 

void timerstart(void) {
  timerstop();

  TIFR &= 0xfe; /* TOV0 to zero ... */
  TCNT0 = 0;

  TIMSK |= (1 << TOIE0);
  
  TCCR0 |= 0x3; /* timer0 an - (011) = prescaler 64 */
}

ISR(TIMER0_OVF_vect, ISR_NOBLOCK) {
  timerstop_safe(inintterupt); /* this could take longer than next tick... */
  
  jiffies++;
  usbPoll();
  
  timerrestart();
  timercont(inintterupt);
  inintterupt=0;
}


USB_PUBLIC uchar usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;
static uchar    replyBuf[8];

    usbMsgPtr = (usbMsgPtr_t)replyBuf;
    
    replyBuf[0] = rq->bRequest;    
    switch (rq->bRequest) {
      
      case PSCMD_CALL_ECHO: {
	replyBuf[1] = PSCMD_RETN_OKAY;
        replyBuf[2] = rq->wValue.bytes[0];
        replyBuf[3] = rq->wValue.bytes[1];
	return 4;
      }
      
      case PSCMD_CALL_GETSTATUS: {
	replyBuf[1] = PSCMD_RETN_OKAY;
	replyBuf[2] = mainstate;
	
	replyBuf[3] = (jiffies >> 0)  & 0xff;
	replyBuf[4] = (jiffies >> 8)  & 0xff;
	replyBuf[5] = (jiffies >> 16) & 0xff;
	replyBuf[6] = (jiffies >> 24) & 0xff;
	replyBuf[7] = 0; /* (jiffies >> 32) & 0xff; */
	return 8;
      }
      
      default: {
	replyBuf[1] = PSCMD_RETN_UNKNOW;
	return 2;
      }
    }
    
    return 0;
}


int main(void)
{
    wdt_enable(WDTO_1S);
    odDebugInit();
    
    DDRD = ~USBMASK;    /* set all pins as outputs except USB */
    PORTD = 0;
    
    timerstop();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */

     _delay_ms(500);
    wdt_reset();
   
    usbDeviceConnect();
    usbInit();
    sei();
    
    timerstart();

    /* MAINPROGRAM */
    mainstate = STATE_BEGINNING;
    
    DDRB |= (1 << PB0);
    for(;;){    /* main event loop */
        wdt_reset();
	PORTB ^= (1 << PB0);
	_delay_ms(25);
     }
    return 0;
}
